package polyglot.ext.polyj.ast;

import polyglot.ext.jl.ast.*;

import polyglot.ext.polyj.types.*;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.visit.*;
import polyglot.util.*;

import java.util.*;

public class InstTypeNode_c extends TypeNode_c implements InstTypeNode
{
    protected TypeNode base;
    protected List params;

    public InstTypeNode_c(Position pos, TypeNode base, List params) {
        super(pos);
        this.base = base;
        this.params = TypedList.copyAndCheck(params, ParamNode.class, true);
    }

    public TypeNode base() {
        return this.base;
    }

    public InstTypeNode base(TypeNode base) {
        InstTypeNode_c n = (InstTypeNode_c) copy();
        n.base = base;
        return n;
    }

    public List params() {
        return this.params;
    }

    public InstTypeNode params(List params) {
        InstTypeNode_c n = (InstTypeNode_c) copy();
        n.params = TypedList.copyAndCheck(params, ParamNode.class, true);
        return n;
    }

    protected InstTypeNode_c reconstruct(TypeNode base, List params) {
        if (base != this.base || ! CollectionUtil.equals(params, this.params)) {
            InstTypeNode_c n = (InstTypeNode_c) copy();
            n.base = base;
            n.params = TypedList.copyAndCheck(params, ParamNode.class, true);
            return n;
        }

        return this;
    }

    public Node visitChildren(NodeVisitor v) {
        TypeNode base = (TypeNode) visitChild(this.base, v);
        List params = visitList(this.params, v);

        return reconstruct(base, params);
    }

    public Node buildTypes(TypeBuilder tb) throws SemanticException {
        PolyJTypeSystem ts = (PolyJTypeSystem) tb.typeSystem();
        return type(ts.unknownType(position()));
    }

    public Node disambiguate(AmbiguityRemover ar) throws SemanticException {
        PolyJTypeSystem ts = (PolyJTypeSystem) ar.typeSystem();
        Type b = (Type) base.type();

        if (! b.isCanonical()) {
            throw new SemanticException(
                "Cannot instantiate from a non-canonical type " + b + ".",
                base.position());
        }

        if (! (b instanceof PolyJClassType)) {
            throw new SemanticException(
                "Cannot instantiate from a non-polymorphic type " + b + ".",
                base.position());
        }

        PolyJClassType t = (PolyJClassType) b;

        List l = new LinkedList();

        for (Iterator i = this.params.iterator(); i.hasNext(); ) {
            ParamNode p = (ParamNode) i.next();
            l.add(p.parameter());
        }

        return ar.nodeFactory().CanonicalTypeNode(position(),
                                                  ts.instantiate(position(),
                                                                 t, l));
    }

    public Node typeCheck(TypeChecker tc) throws SemanticException {
        throw new InternalCompilerError(position(),
            "Cannot type check ambiguous node " + this + ".");
    }

    public Node exceptionCheck(ExceptionChecker ec) throws SemanticException {
        throw new InternalCompilerError(position(),
            "Cannot exception check ambiguous node " + this + ".");
    }

    public void prettyPrint(CodeWriter w, PrettyPrinter tr) {
        print(base, w, tr);

        w.write("[");

        for (Iterator i = this.params.iterator(); i.hasNext(); ) {
            ParamNode p = (ParamNode) i.next();
            print(p, w, tr);
            if (i.hasNext()) {
                w.write(",");
            }
        }

        w.write("]");
    }

    public void translate(CodeWriter w, Translator tr) {
        throw new InternalCompilerError(position(),
            "Cannot translate ambiguous node " + this + ".");
    }

    public String toString() {
        return base + "[...]";
    }
}
